<?php

if ( ARIXWP_CURR_PAGE == 'profile.php' || ARIXWP_CURR_PAGE == 'user-edit.php' ) {

    add_action( 'show_user_profile', 'arixwp_user_profile_fields' );
    add_action( 'edit_user_profile', 'arixwp_user_profile_fields' );
    
    add_action( 'personal_options_update', 'save_arixwp_user_profile_fields' );
    add_action( 'edit_user_profile_update', 'save_arixwp_user_profile_fields' );
    
    $profilemeta = array(
               
                //This is the id applied to the meta box
                'id'         => 'arixwp_profile_meta',
                
                //This is the nonce for this meta
                'nonce_name' => 'arixwp_profile_meta_nonce',
               
                //This is the title that appears on the meta box container
                'title'      => 'Custom Profile Meta',
               
                //Here we define all the fields we want in the meta box
                'fields' => array(	
                    array(
                        'before'	=> __( 'Hey add your custom profile meta here.', THEMENAME ),
                        'name' 		=> __( 'Test Data', THEMENAME ),
                        'desc' 		=> __( 'This is where you add test data.', THEMENAME ),
                        'id' 		=> '_arixwp_profile_test_data',
                        'type' 		=> 'text',
                        'default' 	=> ''
                    ),
                    array(
                        'before'	=> __( 'Hey add your OTHER custom profile meta here.', THEMENAME ),
                        'name' 		=> __( 'Test Data 2', THEMENAME ),
                        'desc' 		=> __( 'This is where you add test data 2.', THEMENAME ),
                        'id' 		=> '_arixwp_profile_test_data2',
                        'type' 		=> 'text',
                        'default' 	=> ''
                    ),
                )
            );
}


function arixwp_user_profile_fields( $user ) {
    
    global $profilemeta;
    
    echo '<input type="hidden" name="' . $profilemeta[ 'nonce_name' ] . '" value="', wp_create_nonce( basename( __FILE__ ) ), '" />';
    
    foreach ( $profilemeta[ 'fields' ] as $field ) {
        
        // get current profile meta data
        $meta = esc_attr( get_the_author_meta( $field[ 'id' ], $user->ID ) );
   
        if ( isset( $field[ 'before' ] ) && $field[ 'before' ] != "" ) {
          echo '<p class="heading">' . $field[ 'before' ] . '</p>';
        }
        
        echo '<label for="'. $field[ 'id' ] .'" class="axLabel">'. $field[ 'name' ]. '</label><p>';
        
        if ( isset( $field[ 'desc' ] ) && $field[ 'desc' ] != "" ) {
          echo '<p class="axText">' . $field[ 'desc' ] . '</p>';
        }
        
        switch ( $field[ 'type' ] ) {
            case 'text':
                echo '<input type="text" name="'. $field[ 'id' ]. '" id="'. $field[ 'id' ] .'" value="'. ( $meta ? $meta : $field[ 'default' ] ) . '" class="axAdmin-input" />'. '<br />';
                break;
            case 'textarea':
                echo '<textarea name="'. $field[ 'id' ]. '" id="'. $field[ 'id' ]. '" cols="60" rows="4" class="axAdmin-input">'. ( $meta ? $meta : $field[ 'default' ] ) . '</textarea>'. '<br />';
                break;
            case 'select':
                echo '<select name="'. $field[ 'id' ] . '" id="'. $field[ 'id' ] . '" class="axAdmin-input">';
                foreach ( $field[ 'options' ] as $option ) {
                    echo '<option '. ( $meta == $option ? ' selected="selected"' : '' ) . '>'. $option . '</option>';
                }
                echo '</select>';
                break;
            case 'radio':
                foreach ( $field[ 'options' ] as $option ) {
                    echo '<input type="radio" name="' . $field[ 'id' ] . '" value="' . $option[ 'value' ] . '"' . ( $meta == $option[ 'value' ] ? ' checked="checked"' : '' ) . ' />' . $option[ 'name' ] . '<br />';
                }
                break;
            case 'checkbox':
                echo '<input type="checkbox" name="' . $field[ 'id' ] . '" id="' . $field[ 'id' ] . '"' . ( $meta ? ' checked="checked"' : '' ) . ' />';
                break;
        }
        echo     '</p>';
    }
}
 
function save_arixwp_user_profile_fields( $user_id ) {
    
    global $profilemeta;
        
    if ( !current_user_can( 'edit_user', $user_id ) ) { return false; }
    
    $_POST[ $profilemeta[ 'nonce_name' ] ] = isset( $_POST[ $profilemeta[ 'nonce_name' ] ] ) ? $_POST[ $profilemeta[ 'nonce_name' ] ] : "";
    
    //Verify nonce
    if ( !wp_verify_nonce( $_POST[$profilemeta[ 'nonce_name' ] ], basename( __FILE__ ) ) ) {
        return false;
    }
    
    foreach ( $profilemeta[ 'fields' ] as $field ) {
        
        $old = esc_attr( get_the_author_meta( $field[ 'id' ], $user_id ) );
        $new = isset( $_POST[ $field[ 'id' ] ] ) ? $_POST[ $field[ 'id' ] ] : '';
       
        if ( $new && $new != $old ) {
            update_user_meta( $user_id, $field[ 'id' ], $new );
        } elseif ( '' == $new && $old ) {
            delete_user_meta( $user_id, $field[ 'id' ], $old );
        }
    }
    
}
